/*----------------------------------------------------------------------------*-

   ttrd2-08c-t0401a-v001a_processor.c (Release 2017-08-22a)

  ----------------------------------------------------------------------------

   Configures Processor in one of the following Modes / States after a reset:

      FAIL_SAFE_S
      NORMAL_M
            
   Provides support for Mode / State changes.

   Provides support for storage, retrival and reporting of PFCs.

-*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*-

  This code is copyright (c) 2014-2017 SafeTTy Systems Ltd.

  This code forms part of a Time-Triggered Reference Design (TTRD) that is 
  documented in the following book: 

   Pont, M.J. (2016) 
   "The Engineering of Reliable Embedded Systems (Second Edition)", 
   Published by SafeTTy Systems Ltd. ISBN: 978-0-9930355-3-1.

  Both the TTRDs and the above book ("ERES2") describe patented 
  technology and are subject to copyright and other restrictions.

  This code may be used without charge: [i] by universities and colleges on 
  courses for which a degree up to and including 'MSc' level (or equivalent) 
  is awarded; [ii] for non-commercial projects carried out by individuals 
  and hobbyists.

  Any and all other use of this code and / or the patented technology 
  described in ERES2 requires purchase of a ReliabiliTTy Technology Licence:
  http://www.safetty.net/technology/reliabilitty-technology-licences

  Please contact SafeTTy Systems Ltd if you require clarification of these 
  licensing arrangements: http://www.safetty.net/contact

   CorrelaTTor, DuplicaTTor, MoniTTor, PredicTTor, ReliabiliTTy, SafeTTy, 
   SafeTTy Systems and WarranTTor are registered trademarks or trademarks 
   of SafeTTy Systems Ltd in the UK and other countries.

-*----------------------------------------------------------------------------*/

// Processor Header
#include "../main/main.h"

// Support for reporting PFCs
#include "ttrd2-05a-t0401a-v001a_pfc_reporting.h"

// Tasks
#include "../tasks/ttrd2-05a-t0401a-v001a_iwdt_task.h"
#include "../tasks/ttrd2-05a-t0401a-v001a_uart2_buff_o_task.h"
#include "../tasks/ttrd2-08c-t0401a-v001a_heartbeat_oru_task.h"

// ------ Private Duplicated Variables ('File Global') -----------------------

// The current system Mode / State
static eProc_MoSt Processor_MoSt_g;
static uint32_t   Processor_MoSt_ig;

// ------ Private function prototypes (declarations) -------------------------

      static void PROCESSOR_Identify_Reqd_MoSt(void);
      static void PROCESSOR_Configure_Reqd_MoSt(void);

      static void PROCESSOR_Set_MoSt(const eProc_MoSt MOST);

static eProc_MoSt PROCESSOR_Retrieve_Reqd_MoSt(void);

  static uint32_t PROCESSOR_Retrieve_PFC(void);

/*----------------------------------------------------------------------------*-

  PROCESSOR_Init()

  Wrapper for system startup functions.
  
  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Init(void)
   {
   PROCESSOR_Identify_Reqd_MoSt();
   PROCESSOR_Configure_Reqd_MoSt();
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Identify_Reqd_MoSt()

  Identify the cause of the system reset and set the system mode accordingly.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     Processor_MoSt_g (W)

  MCU HARDWARE:
     Registers indicating cause of reset (R).

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Identify_Reqd_MoSt(void)
   {
   uint32_t Fault_code;

   if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST))
      {
      // WDT reset.  Could be the result of WDT test.
      // => we need to check the mode and fault codes
      Fault_code = PROCESSOR_Retrieve_PFC();

      if (Fault_code == PFC_NO_FAULT)
         {
         // WDT triggered during testing => retrieve required mode 
         PROCESSOR_Set_MoSt(PROCESSOR_Retrieve_Reqd_MoSt());      
         }
      else
         {
         // Not a test => Fail Safe
         PROCESSOR_Set_MoSt(FAIL_SAFE_S);
         }
      }
   else if (RCC_GetFlagStatus(RCC_FLAG_SFTRST))
      {
      // Software reset (incl. debugger)
      // Under normal operation this will be used for planned mode changes
      // => we need to check the required mode 
      PROCESSOR_Set_MoSt(PROCESSOR_Retrieve_Reqd_MoSt());
      }
   else if (RCC_GetFlagStatus(RCC_FLAG_PORRST))
      {
      // Power-On Reset
      // Start in 'NORMAL_M' mode
      PROCESSOR_Set_MoSt(NORMAL_M);

      // Clear SMR count
      PROCESSOR_Store_SMR_Count(0);
      }
   else if (RCC_GetFlagStatus(RCC_FLAG_PINRST))
      {
      // 'Reset button' pressed on Nucleo board
      // Here we treat this is the same way as a POR
      PROCESSOR_Set_MoSt(NORMAL_M);

      // Clear SMR count
      PROCESSOR_Store_SMR_Count(0);
      }
   else
      {
      // Other possible reset causes
      // Low-power management reset / Windowed WDT / Brown-out
      // In all cases we enter fail-safe state
      PROCESSOR_Set_MoSt(FAIL_SAFE_S);
      }   
  
   // Clear all reset flags
   RCC_ClearFlag();
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Configure_Reqd_MoSt()

  Configure the system in the required mode.  

  PARAMETERS:
     None.

  LONG-TERM DATA:
     System_node_G (R)

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.
   
-*----------------------------------------------------------------------------*/
void PROCESSOR_Configure_Reqd_MoSt(void)
   {
   // Prepare UART-USB interface
   UART2_BUF_O_Init(230400);
   UART2_BUF_O_Write_String_To_Buffer("\nStarting up\n");

   UART2_BUF_O_Write_String_To_Buffer("SMR Count : ");
   UART2_BUF_O_Write_Number02_To_Buffer(PROCESSOR_Retrieve_SMR_Count());
   UART2_BUF_O_Write_String_To_Buffer("\n");
   UART2_BUF_O_Send_All_Data();

   // Check data integrity (Processor_MoSt_g)
   if (Processor_MoSt_ig != ~((uint32_t) Processor_MoSt_g))
      {
      // Data-integrity problem
      Processor_MoSt_g = FAIL_SAFE_S;
      Processor_MoSt_ig = ~((uint32_t) FAIL_SAFE_S);
      }

   switch (Processor_MoSt_g)
      {
      default:          // Default to "FAIL_SAFE_S"
      case FAIL_SAFE_S:
         {
         UART2_BUF_O_Write_String_To_Buffer("\nFAIL_SAFE_S\n");
         UART2_BUF_O_Send_All_Data();

         // Trigger "fail safe" behaviour
         PROCESSOR_Perform_Safe_Shutdown(PROCESSOR_Retrieve_PFC());

         break;
         }

      case NORMAL_M:
         {
         // Store default state and fault code
         PROCESSOR_Store_Reqd_MoSt(FAIL_SAFE_S);
         PROCESSOR_Store_PFC(PFC_UNSPECIFIED_FAULT);

         // Report Mode / State (for demo purposes only)
         UART2_BUF_O_Write_String_To_Buffer("NORMAL_M\n");

         // Set up scheduler for 5 ms ticks
         SCH_Init_Microseconds(5000);

         // Set up WDT 
         // Timeout is parameter * 125 s: param 80 => ~10 ms
         // NOTE: WDT driven by RC oscillator - timing varies with temperature            
         WATCHDOG_Init(80);

         // Prepare for heartbeat task
         HEARTBEAT_ORU_Init();
         
         // Store the GPIO register configuration
         // Check in the Heartbeat task 
         // - and any tasks that perform safety-related IO
         REG_CONFIG_CHECKS_GPIO_Store();

         // Add tasks to schedule.
         // Parameters are:
         // A. Task name
         // B. Initial delay / offset (in Ticks)
         // C. Task period (in Ticks): Must be > 0
         //
         //           A                      B   C    
         SCH_Add_Task(WATCHDOG_Update,       0,  1);     // iWDT
         SCH_Add_Task(HEARTBEAT_ORU_Update,   0,  200);   // Heartbeat
         SCH_Add_Task(UART2_BUF_O_Update,    0,  1);     // UART2

         // Feed the watchdog
         WATCHDOG_Update();

         break;
         }
      }
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Set_MoSt()

  Set the value of 'Processor_MoSt_g' (Duplicated Variable). 
   
  PARAMETERS:
     MOST : Value to be assigned.

  LONG-TERM DATA:
     Processor_MoSt_g (W)
     Processor_MoSt_ig (W)
   
  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
static void PROCESSOR_Set_MoSt(const eProc_MoSt MOST)
   {
   Processor_MoSt_g = MOST;
   Processor_MoSt_ig = ~((uint32_t)MOST);
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Get_MoSt()

  Returns the value of 'Processor_MoSt_g' (Duplicated Variable). 
   
  PARAMETERS:
     None.

  LONG-TERM DATA:
     Processor_MoSt_g (R)
     Processor_MoSt_ig (R)
   
  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     Checks for data corruption (=> shutdown)

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     The value of 'Processor_MoSt_g' (Duplicated Variable). 

-*----------------------------------------------------------------------------*/
eProc_MoSt PROCESSOR_Get_MoSt(void)
   {
   // Check data integrity (file-global variable)
   // Shut down if corruption is detected.
   // Other design solutions may be more appropriate in your system.
   if (Processor_MoSt_ig != ~((uint32_t) Processor_MoSt_g))
      {
      PROCESSOR_Perform_Safe_Shutdown(PFC_LONG_TERM_DATA_CORRUPTION);
      }
 
   return Processor_MoSt_g;
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Change_MoSt()

  Force software reset, which will (at minimum) cause a 'Same-Mode Reset'.

  Note.  Before calling this function, PROCESSOR_Store_Reqd_MoSt() will 
  usually be called (to indicate the required new Mode / State). 

  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.
       
  MCU HARDWARE:
      NVIC.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Change_MoSt(void)
   {
   // Trigger software reset
   NVIC_SystemReset();       
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Store_Reqd_MoSt()

  Store the required (next) Mode / State in a backup register.

  Stored as a Duplicated Variable.

  PARAMETERS:
     The required Mode / State.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Store_Reqd_MoSt(const eProc_MoSt REQD_MoSt)
   {
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
   PWR_BackupAccessCmd(ENABLE);

   RTC->BKP0R =  (uint32_t)REQD_MoSt;
   RTC->BKP1R = ~(uint32_t)REQD_MoSt;
  
   PWR_BackupAccessCmd(DISABLE);
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, DISABLE);
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Retrieve_Reqd_MoSt()

  Retrieve the required (next) Mode / State from the backup registers.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     The data are stored in the form of a Duplicated Variable.
     If the copies don't match, FAIL_SAFE_S is returned.

  RETURN VALUE:
     The stored Mode / State (or FAIL_SAFE_S - see above).

-*----------------------------------------------------------------------------*/
eProc_MoSt PROCESSOR_Retrieve_Reqd_MoSt(void)
   {
   eProc_MoSt return_value;
  
   if (RTC->BKP0R != ~(RTC->BKP1R))
      {
      // Backup register content has been corrupted
      return_value = FAIL_SAFE_S;
      }
  else
     {
    // Backup register content seems to be correct
    return_value = (eProc_MoSt) RTC->BKP0R;
    }    
   
   return return_value;
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Store_PFC()

  Store a Processor Fault Code in backup registers.

  Stored as a Duplicated Variable.

  PARAMETERS:
     The PFC.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Store_PFC(const uint32_t FAULT_CODE)
   {
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
   PWR_BackupAccessCmd(ENABLE);

   RTC->BKP2R =  FAULT_CODE;
   RTC->BKP3R = ~FAULT_CODE;
  
   PWR_BackupAccessCmd(DISABLE);
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, DISABLE);
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Retrieve_Reqd_MoSt()

  Retrieve the Processor Fault Code from the backup registers.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     The data are stored in the form of a Duplicated Variable.
     If the copies don't match, PFC_UNKNOWN_FAULT is returned.

  RETURN VALUE:
     The stored PFC (or PFC_UNKNOWN_FAULT - see above).

-*----------------------------------------------------------------------------*/
uint32_t PROCESSOR_Retrieve_PFC(void)
   {
   uint32_t return_value;
   
   if (RTC->BKP2R != ~(RTC->BKP3R))
      {
      // Backup register content has been corrupted
      return_value = PFC_UNKNOWN_FAULT;  
      }
   else
      {
      // Backup register content seems to be correct
      return_value = RTC->BKP2R;
      }
  
   return return_value;
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Store_SMR_Count()

  Store the number of SMRs that have been performed since the last POR 
  in a backup register.

  Stored as a Duplicated Variable.

  PARAMETERS:
     The current count.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Store_SMR_Count(const uint32_t COUNT)
   {
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
   PWR_BackupAccessCmd(ENABLE);

   RTC->BKP4R =  (uint32_t)COUNT;
   RTC->BKP5R = ~(uint32_t)COUNT;
  
   PWR_BackupAccessCmd(DISABLE);
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, DISABLE);
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Retrieve_SMR_Count()

  Retrieve the stored SMR count from the backup registers.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     The data in the backup registers.
       
  MCU HARDWARE:
     Two 32-bit backup registers.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     The data are stored in the form of a Duplicated Variable.
     If the copies don't match, 0 is returned.

  RETURN VALUE:
     The stored count (or 0).

-*----------------------------------------------------------------------------*/
uint32_t PROCESSOR_Retrieve_SMR_Count(void)
   {
   uint32_t return_value;

      if (RTC->BKP4R != ~(RTC->BKP5R))
         {
         // Backup register content has been corrupted, 
         // or no count has been stored previously.
         // In this demo, we assume that this is the first run.
         return_value = 0;         
         }
      else
         {
         // Backup register content seems to be correct
         return_value = (uint32_t) RTC->BKP4R;
         }    

   return return_value;
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Perform_Safe_Shutdown()

  Attempt to move the system into a safe ('silent') state.

  If WDT is running, we aim to reset the processor and call this function again
  (without the WDT running), in order to keep this function very simple,
  and reduce the risks that we will end up in a repeated sequence of resets.

  PARAMETERS:
     The Processor Fault Code.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.
   
-*----------------------------------------------------------------------------*/
void PROCESSOR_Perform_Safe_Shutdown(const uint32_t PFC)
   {
   uint32_t WDT_state = WATCHDOG_Get_State();

   // Check if the iWDT is running
   if (WDT_state == WDT_RUNNING)
      {
      // Feed the WDT
      WATCHDOG_Update();

      // Reset (with the aim of disabling the iWDT)
      PROCESSOR_Store_Reqd_MoSt(FAIL_SAFE_S);
      PROCESSOR_Store_PFC(PFC);
      PROCESSOR_Change_MoSt();
      }

   if (WDT_state == WDT_UNKNOWN_STATE)
      {
      // Reset (with the aim of disabling the iWDT, just in case)
      PROCESSOR_Store_Reqd_MoSt(FAIL_SAFE_S);
      PROCESSOR_Store_PFC(PFC);
      PROCESSOR_Change_MoSt();
      }

   // If we've got this far, we assume that the iWDT is *not* running

   // Set up 'Heartbeat' LED pin to report the PFC
   REPORT_PFC_Init();

   while(1)
      {
      // 0.5-second delay (max timeout is ~65 ms => we use 50 ms x 10)
      for (uint32_t i = 0; i <= 10; i++)
         {
         TIMEOUT_T3_USEC_Start();
         while (COUNTING == TIMEOUT_T3_USEC_Get_Timer_State(50000));
         }       

      // Report PFC
      REPORT_PFC_Update(PFC);
      }
   }

/*----------------------------------------------------------------------------*-
  ------------------------------ END OF FILE ---------------------------------
-*----------------------------------------------------------------------------*/
